import type * as T from '@traversable/registry';
import { t } from '@traversable/schema';
import * as JsonSchema from './jsonSchema.js';
type JsonSchema = import('./jsonSchema.js').JsonSchema;
export declare namespace Recursive {
    const toJsonSchema: T.Algebra<t.Free, () => JsonSchema>;
    const fromJsonSchema: T.Algebra<JsonSchema.Free, t.Schema>;
}
/**
 * ## {@link fromJsonSchema `fromJsonSchema`}
 *
 * Recursively convert a JSON schema into a traversable schema.
 *
 * __Roundtrip property:__
 *
 * Thoroughly tested using randomly generated schemas to be invertible when paired
 * with {@link toJsonSchema} without any loss of information.
 */
export declare const fromJsonSchema: <S extends JsonSchema.JsonSchema>(term: S) => t.LowerBound;
/**
 * ## {@link toJsonSchema `toJsonSchema`}
 *
 * Recursively convert a traversable schema into its JSON schema representation.
 *
 * __Roundtrip property:__
 *
 * Thoroughly tested using randomly generated schemas to be invertible with
 * {@link fromJsonSchema `fromJsonSchema`} without any loss of information.
 *
 * __Note:__ If you find yourself reaching for {@link toJsonSchema `toJsonSchema`},
 * keep in mind that by installing the `@traversable/schema-to-json-schema` package,
 * all of the built-in schemas from the `@traversable/schema` package come
 * equipped with a `.toJsonSchema` getter method.
 *
 * Prefer using the `.toJsonSchema` method if possible -- not only is it
 * easier to use, but it's also more performant, since the implementation
 * is not recursive.
 */
export declare const toJsonSchema: <S extends t.LowerBound>(term: S) => () => JsonSchema.JsonSchema;
export {};
//# sourceMappingURL=recursive.d.ts.map